<?php 
namespace Concrete\Package\GntWordCloudify\Block\GntWordCloudify;

defined('C5_EXECUTE') or die('Access Denied.');

/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2013 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
use Concrete\Core\Block\BlockController;
use Page;
use PageList;
use Area;
use CollectionAttributeKey;
use Log;
use Concrete\Package\GntWordCloudify\Src\AssetButler;
use AssetList;
use Concrete\Core\Asset\JavascriptAsset;
use Concrete\Core\Asset\Asset;
use Concrete\Attribute\Select\OptionList;
use Package;

/*
 * Porting TODO:
 * - check the attribute usage
 * - merge the view stuff
 */


class Controller extends BlockController
{

    protected $btTable = "btGntWordCloudify";
    protected $btInterfaceWidth = "700";
    protected $btInterfaceHeight = "450";
    protected $btCacheBlockOutput = false;
    protected $btCacheBlockOutputOnPost = false;
    protected $btCacheBlockOutputForRegisteredUsers = false;
    protected $btDefaultSet = 'social';

    private $_wordList = null;

    public function getBlockTypeDescription()
    {
        return t("Creates an SVG words cloud");
    }
    public function getBlockTypeName()
    {
        return t("Word Cloudify");
    }

    public function save($args)
    {
        $zeroify = array( 'useLocalD3Js', 'loadD3js', 'precalc', 'loadEffect', 'addCustomVar',
                          'shuffle', 'uppercaseWords', 'caseInsensitive', 'enforceSize', 'cacheTags', 'redirect', );
        $intify = array(
            'width', 'height', 'scale', 'orientationsNumbers',
            'minAngle', 'maxAngle', 'minSize', 'maxSize', 'maxWords',
        );

        foreach ($zeroify as $key) {
            $args[$key] = isset($args[$key]) ? $args[$key] : 0;
        }
        foreach ($intify as $key) {
            $args[$key] = intval($args[$key]);
        }

        // Since we are going to compute some cache here, we need to set the object as it will be after save

        foreach ($args as $key => $val) {
            $this->$key = $val;
        }

        $source = $args['wordsSource'];
        if (self::SOURCE_MANUAL == $source || ($args['cacheTags'] && self::SOURCE_MANUAL_ADVANCED != $source)) {
            if (self::SOURCE_MANUAL == $source) {
                $words = $this->_getTagsFromText($args['words']);
            } else {
                $words = $this->_getWordsFromSite();
            }
            $csv = $this->_wordsToCSV($words);
            $this->tags = $csv;
            $args['tags'] = $csv;
        }
        parent::save($args);
    }

    # search types :
    const SEARCH_ALL           = 0;
    const SEARCH_CHILDS        = 1;
    const SEARCH_UNDER         = 2;
    const SEARCH_ALL_CHILDS    = 3;
    const SEARCH_ALL_UNDER     = 4;
    const SEARCH_CURRENT_PAGE  = 5;
    const SEARCH_SPECIFIC_PAGE = 6;

    protected static $_searchTypes = null;

    public static function getSearchTypes()
    {
        if (is_null(self::$_searchTypes)) {
            self::$_searchTypes = array(
                self::SEARCH_ALL           => t('All site pages'),
                self::SEARCH_CHILDS        => t('Pages directly under this page'),
                self::SEARCH_ALL_CHILDS    => t('All pages under this page'),
                self::SEARCH_UNDER         => t('Pages directly under another page (to be selected)'),
                self::SEARCH_ALL_UNDER     => t('All Pages under another page (to be selected)'),
                self::SEARCH_CURRENT_PAGE  => t("Only the current page"),
                self::SEARCH_SPECIFIC_PAGE => t("Only a specific page"),
            );
        }

        return self::$_searchTypes;
    }

    # word sources :

    const SOURCE_TAGS               = 0;
    const SOURCE_DESC               = 1;
    const SOURCE_CONTENT            = 2;
    const SOURCE_MANUAL             = 3;
    const SOURCE_MANUAL_ADVANCED    = 4; // XXX Warning this one is hardcoded in auto.js (hack)

    protected static $_wordSources = null;
    public static function getWordSources()
    {
        if (is_null(self::$_wordSources)) {
            self::$_wordSources = array(
                self::SOURCE_TAGS            => t("Page Tags"),
                self::SOURCE_DESC            => t('Page Summary'),
                self::SOURCE_CONTENT         => t('Page Main Area Content'),
                self::SOURCE_MANUAL          => t('Manually entered text'),
                self::SOURCE_MANUAL_ADVANCED => t('Manually entered list (advanced)'),
            );
        }

        return self::$_wordSources;
    }

    # occurence  / weight scale
    const SCALE_LINEAR              = 0;
    const SCALE_SQRT                = 1;
    const SCALE_LOG                 = 2;
    const SCALE_RAND                = 3;

    protected static $_scaleList = null;
    public static function getScaleList()
    {
        if (is_null(self::$_scaleList)) {
            self::$_scaleList = array(
                self::SCALE_LINEAR          => t('Linear'),
                self::SCALE_SQRT            => t('Square root'),
                self::SCALE_LOG             => t('Logarithm'),
                self::SCALE_RAND            => t('Random'),
            );
        }

        return self::$_scaleList;
    }

    # Coloring types
    const COLORING_SCALE10    = 'category10';
    const COLORING_SCALE20    = 'category20';
    const COLORING_SCALE20B   = 'category20b';
    const COLORING_SCALE20C   = 'category20c';
    const COLORING_GRADIENT   = 'gradient';
    const COLORING_MONOCHROME = 'monochrome';
    const COLORING_BICOLOR    = 'bicolor';
    const COLORING_INHERIT    = 'inherit';

    private static $_coloringList = null;
    public static function getColoringList()
    {
        if (is_null(self::$_coloringList)) {
            self::$_coloringList = array(
                self::COLORING_SCALE20    => t("D3 category 20"),
                self::COLORING_SCALE20B   => t("D3 category 20B"),
                self::COLORING_SCALE20C   => t("D3 category 20C"),
                self::COLORING_SCALE10    => t("D3 category 10"),
                self::COLORING_GRADIENT   => t("Gradient"),
                self::COLORING_MONOCHROME => t("Monochrome"),
                self::COLORING_BICOLOR    => t("Bicolored"),
                self::COLORING_INHERIT    => t("Inherit from css"),
            );
        }

        return self::$_coloringList;
    }

    const COLORING_SCALE_ORDINAL = 'ordinal';
    const COLORING_SCALE_SIZE    = 'size';
    private static $_coloringScales = null;
    public static function getColoringScales()
    {
        if (is_null(self::$_coloringScales)) {
            self::$_coloringScales = array(
                self::COLORING_SCALE_ORDINAL => t("word index"),
                self::COLORING_SCALE_SIZE    => t("word size (weight)"),
            );
        }

        return self::$_coloringScales;
    }

    const TARGET_SEARCH_NOSEARCH    = 0;
    const TARGET_SEARCH_QUERY       = 1;
    const TARGET_SEARCH_TAGS        = 2;
    const TARGET_SEARCH_CUSTOM      = 3;
    private static $_targetTypes = null;

    public static function getTargetTypes()
    {
        if (! self::$_targetTypes) {
            self::$_targetTypes = array(
                self::TARGET_SEARCH_QUERY    => t('Search in site (query)'),
                self::TARGET_SEARCH_TAGS     => t('Search in page tags'),
                self::TARGET_SEARCH_CUSTOM   => t('Use a custom variable'),
                self::TARGET_SEARCH_NOSEARCH => t('No search'),
            );
        }

        return self::$_targetTypes;
    }

    const REDIRECT_TO_PAGE  = 0;
    const REDIRECT_TO_URL   = 1;
    private static $_redirectTypes = null;
    public static function getRedirectTypes()
    {
        if (!self::$_redirectTypes) {
            self::$_redirectTypes = array(
                self::REDIRECT_TO_PAGE => t('Redirect to page'),
                self::REDIRECT_TO_URL  => t('Redirect to url'),
            );
        }

        return self::$_redirectTypes;
    }

    protected function _getWordUrl($word, $id = null)
    {
        static $targetBaseUrl = null;
        static $sep = '?';
        static $var = null;
        static $tagOptions = null;

        if (isset($word['href'])) {
            return $word['href'];
        }
        if (!$this->redirect) {
            return '';
        }

        if (is_null($targetBaseUrl)) {
            $targetBaseUrl = $this->link;
            if (self::REDIRECT_TO_PAGE == $this->redirectType) {
                $nav = \Core::make('helper/navigation');
                $id = $this->targetPage;

                $page = Page::getByID($id);
                $targetBaseUrl = '';

                if ($page && !$page->isError()) {
                    $targetBaseUrl = $nav->getCollectionURL($page);
                }
            }
            if (strpos($targetBaseUrl, '?') !== false) {
                $sep = '&';
            }
        }
        if (!$targetBaseUrl) {
            return '';
        }

        if (is_null($var)) {
            $vars = array(
                self::TARGET_SEARCH_NOSEARCH => '',
                self::TARGET_SEARCH_QUERY => 'query',
                self::TARGET_SEARCH_CUSTOM => $this->customVar,
            );
            if (isset($vars[$this->targetType])) {
                $var = $vars[$this->targetType];
            } else {
                $ak = CollectionAttributeKey::getByHandle('tags');
                $id = $ak->getAttributeKeyID();
                $var = "akID[$id][atSelectOptionID][]";
            }
        }

        $extraVar = '';
        if (self::TARGET_SEARCH_NOSEARCH != $this->targetType && $this->addCustomVar) {
            $extraVar = $this->customVar;
        }

        if (!$var && ! $extraVar) {
            return $targetBaseUrl;
        }

        $val = $word['word'];
        if ($this->targetType == self::TARGET_SEARCH_TAGS) {
            if (!is_null($id)) {
                $val = $id;
            } else {
                $w = $word['word'];
                if (is_null($tagOptions)) {
                    $tagOptions = array();

                    $ak = CollectionAttributeKey::getByHandle('tags');
                    $controller = $ak->getAttributeType()->getController();
                    $controller->setAttributeKey($ak);
                    $items = $controller->getOptions();
                    foreach ($items as $item) {
                        $tagOptions[$item->getSelectAttributeOptionValue()] = $item->getSelectAttributeOptionID();
                    }
                }
                $val = isset($tagOptions[$w]) ? $tagOptions[$w] : null;
            }
        }
        if (is_null($val) && ! $extraVar) {
            return $targetBaseUrl;
        }

        $s = $sep;
        $url = $targetBaseUrl;
        if (!is_null($val) && $var) {
            $url = "${url}${s}${var}=".htmlentities($val);
            $s = '&';
        }

        if (!$extraVar) {
            return $url;
        }

        return "$url$s$extraVar=".htmlentities($word[word]);
    }

    // Color scale application/calculation
    protected function _colorPalette($scale, $idx, $size, $max)
    {
        $ordinal = $this->scaleType == self::COLORING_SCALE_ORDINAL;
        if ($ordinal) {
            return $scale[$idx%count($scale)];
        }

        return $scale[round(((float) $size/$max)*(count($scale)-1))];
    }

    public static function getScaleColors($type)
    {
        static $scales = array(

            self::COLORING_SCALE10 => array(
                "#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd",
                "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf",
            ),

            self::COLORING_SCALE20 => array(
                "#1f77b4", "#aec7e8", "#ff7f0e", "#ffbb78", "#2ca02c",
                "#98df8a", "#d62728", "#ff9896", "#9467bd", "#c5b0d5",
                "#8c564b", "#c49c94", "#e377c2", "#f7b6d2", "#7f7f7f",
                "#c7c7c7", "#bcbd22", "#dbdb8d", "#17becf", "#9edae5",
            ),
            self::COLORING_SCALE20B => array(
                "#393b79", "#5254a3", "#6b6ecf", "#9c9ede",
                "#637939", "#8ca252", "#b5cf6b", "#cedb9c",
                "#8c6d31", "#bd9e39", "#e7ba52", "#e7cb94",
                "#843c39", "#ad494a", "#d6616b", "#e7969c",
                "#7b4173", "#a55194", "#ce6dbd", "#de9ed6",
            ),
            self::COLORING_SCALE20C => array(
                "#3182bd", "#6baed6", "#9ecae1", "#c6dbef",
                "#e6550d", "#fd8d3c", "#fdae6b", "#fdd0a2",
                "#31a354", "#74c476", "#a1d99b", "#c7e9c0",
                "#756bb1", "#9e9ac8", "#bcbddc", "#dadaeb",
                "#636363", "#969696", "#bdbdbd", "#d9d9d9",
            ),
            self::COLORING_GRADIENT => array(
                "#000", "#fff", "#000", "#fff",
                "#000", "#fff", "#000", "#fff",
                "#000", "#fff", "#000", "#fff",
                "#000", "#fff", "#000", "#fff",
                "#000", "#fff", "#000", "#fff",
            ),

        );

        if (!isset($scales[$type])) {
            return array();
        }

        return $scales[$type];
    }

    protected function _scale_category10($idx, $size, $max)
    {
        return $this->_colorPalette(self::getScaleColors(self::COLORING_SCALE10), $idx, $size, $max);
    }
    protected function _scale_category20($idx, $size, $max)
    {
        return $this->_colorPalette(self::getScaleColors(self::COLORING_SCALE20), $idx, $size, $max);
    }
    protected function _scale_category20b($idx, $size, $max)
    {
        return $this->_colorPalette(self::getScaleColors(self::COLORING_SCALE20B), $idx, $size, $max);
    }
    protected function _scale_category20c($idx, $size, $max)
    {
        return $this->_colorPalette(self::getScaleColors(self::COLORING_SCALE20C), $idx, $size, $max);
    }

    public static function rgbToInt($hex)
    {
        if ($hex[0] == '#') {
            $hex = substr($hex, 1);
        }

        return array( hexdec(substr($hex, 0, 2)), hexdec(substr($hex, 2, 2)),hexdec(substr($hex, 4, 2)));
    }
    protected function _scale_gradient($idx, $size, $max)
    {
        static $start = null;
        static $end = null;
        if (is_null($start)) {
            $start = self::rgbToInt($this->gradientStart);
            $end = self::rgbToInt($this->gradientEnd);
        }

        $ordinal = $this->scaleType == self::COLORING_SCALE_ORDINAL;

        $s = 0;
        if ($ordinal) {
            $s = (float) ($idx%21)/20.0;
        } else {
            $s = (float) $size / (float) $max;
        }

        $c = '#';
        $c .= sprintf("%02x", $start[0] * (1-$s) + $end[0] * $s);
        $c .= sprintf("%02x", $start[1] * (1-$s) + $end[1] * $s);
        $c .= sprintf("%02x", $start[2] * (1-$s) + $end[2] * $s);

        return $c;
    }
    protected function _scale_monochrome($idx, $size, $max)
    {
        return $this->gradientStart;
    }
    protected function _scale_bicolor($idx, $size, $max)
    {
        static $scale = null;
        if (is_null($scale)) {
            $scale = array( $this->gradientStart, $this->gradientEnd );
        }

        return $this->_colorPalette($scale, $idx, $size, $max);
    }
    protected function _scale_inherit($idx, $size, $max)
    {
        return '';
    }

    protected function _getPageTagsWords($page, $words)
    {
        if (!$words) {
            $words = array();
        }

        $list = $page->getAttribute('tags');
        if (!$list || !($list instanceof OptionList) || $list->count() == 0) {
            return $words;
        }

        foreach ($list->getOptions() as $item) {
            $word = $item->getSelectAttributeOptionDisplayValue();
            if (!isset($words[$word])) {
                $words[$word] = array( 'word' => $word, 'count' => 0 );
            }
            $words[$word]['count']++;
        }

        return $words;
    }

    protected function _getPageDescWords($page, $words = array())
    {
        return $this->_getWordsFromText($page->getCollectionDescription(), $words);
    }

    protected function _getPageContentWords($page, $words = array())
    {
        static $recurs = false;

        if ($recurs) {
            return $words;
        }
        $recurs = true;

        ob_start();
        $a = new Area('Main');
        $a->disableControls();
        $a->display($page);
        $text = ob_get_clean();

        // remove scripts tags :
        $text = preg_replace(",\n,", ' ', $text);                                    // strip lines
        $text = preg_replace(",<\s*script[^>]*>.+</\s*script[^>]*>,i", "", $text); // strip js scripts
        $text = strip_tags($text);                                                // strip html tags

        $recurs = false;

        return $this->_getWordsFromText($text, $words);
    }

    protected function _getPageArray()
    {
        //$list->filterByAttribute( array( 'total_votes' => $akHandle ), null, 'is not' );
        //
        $searchType = $this->searchType;
        $baseNode = $this->baseNode;

        $list = new PageList();
        switch ($searchType) {
            case self::SEARCH_SPECIFIC_PAGE :
                $page = Page::getByID($baseNode);
                if (!$page || $page->isError()) {
                    return array();
                }

                return array( $page );

            case self::SEARCH_CURRENT_PAGE :
                return array( Page::getCurrentPage() );

            case self::SEARCH_CHILDS :
                $baseNode = Page::getCurrentPage()->getCollectionID();
                $list->filterByParentID($baseNode);
                break;

            case self::SEARCH_UNDER :
                $list->filterByParentID($baseNode);
                break;

            case self::SEARCH_ALL_CHILDS :
                $path = Page::getCurrentPage()->getCollectionPath();
                $list->filterByPath($path);
                break;

            case self::SEARCH_ALL_UNDER :
                $path = Page::getByID($baseNode)->getCollectionPath();
                $list->filterByPath($path);
                break;
        }

        $list->filter('cvName', '', '!=');

        $pType = $this->filterPageType;
        if ($pType) {
            $list->filterByCollectionTypeID($pType);
        }

        return $list->get();
    }

    public function _getWordsFromSite()
    {

        // sanity check :
        $getTextFuncs = array(
            self::SOURCE_TAGS    => '_getPageTagsWords',
            self::SOURCE_DESC    => '_getPageDescWords',
            self::SOURCE_CONTENT => '_getPageContentWords',
        );
        $getTextFunc = $getTextFuncs[$this->wordsSource];
        if (!$getTextFunc) {
            return;
        }

        $words = array();
        $pages = $this->_getPageArray();
        foreach ($pages as $page) {
            $words = $this->$getTextFunc($page, $words);
        }

        return $this->_computeWordList($words);
    }

    protected static function _scaleLinear($x, $max)
    {
        $div = (float) $max;

        return (float) $x / ($div ? $div : 1);
    }
    protected static function _scaleLog($x, $max)
    {
        $div = log($max);

        return log($x) / ($div ? $div : 1);
    }
    protected static function _scaleSqrt($x, $max)
    {
        $div = sqrt($max);

        return sqrt($x) / ($div ? $div : 1);
    }
    protected static function _scaleRandom($x, $max)
    {
        return (float) rand()/(float) getrandmax();
    }

    protected function _getTagsFromText($text)
    {
        $words = $this->_getWordsFromText($text);

        return $this->_computeWordList($words);
    }

    protected function _getWordsFromText($text, $words = array())
    {
        $w = str_replace(' ', "\n", $text);

        $caseInsensitive = $this->caseInsensitive;
        $upperCase = $this->uppercaseWords;
        $cnt = array();
        foreach (preg_split("/[\n,\. ?;]/", $w) as $word) {
            $word = trim($word);
            if ($word == '') {
                continue;
            }
            if ($caseInsensitive) {
                $word = $upperCase ? strtoupper($word) : strtolower($word);
            }
            $cnt[$word] = intval($cnt[$word]) + 1;
        }

        if (!$words) {
            $words = array();
        }
        foreach ($cnt as $word => $count) {
            if (! $words[$word ]) {
                $words[$word] = array( 'word' => $word, 'count' => $count );
            }
        }

        return $words;
    }

    protected function _getWordList()
    {
        $words = array();
        $source = $this->wordsSource;

        if ($this->cacheTags) {
            $source = self::SOURCE_MANUAL;
        }

        switch ($source) {
            case self::SOURCE_MANUAL:
            case self::SOURCE_MANUAL_ADVANCED :
                $words = $this->_csvToWords($this->tags);
                break;

            default:
                $words = $this->_getWordsFromSite();
                break;
        }

        if ($this->shuffle) {
            shuffle($words);
        }

        return $words;
    }

    protected function _computeWordList(array $words)
    {
        if (!count($words)) {
            return array();
        }

        // compute the maximum Occurrences :
        $max = 0;
        foreach ($words as $tuple) {
            if ($tuple['count'] > $max) {
                $max = $tuple['count'];
            }
        }
        if ($max == 0) {
            $max = 1;
        }

        // Scale the words :
        $scales = array(
            self::SCALE_LINEAR => '_scaleLinear',
            self::SCALE_SQRT   => '_scaleSqrt',
            self::SCALE_LOG    => '_scaleLog',
            self::SCALE_RAND   => '_scaleRandom',
        );
        $scale      = $scales[$this->scale];
        $colorScale = "_scale_".$this->coloringType;
        if (!method_exists($this, $colorScale)) {
            Log::warning("Unknown color scale $colorScale");
            $colorScale = '_scale_category20';
        }

        $minSize = (float) $this->minSize;
        $maxSize = (float) $this->maxSize;

        $csvStruct = self::getCSVStruct();

        // compute exclude list
        $excludeList =  array();
        if ($this->excludeList) {
            $list = preg_split('/,/', $this->excludeList);
            foreach ($list as $word) {
                $word = trim($word);
                if ($caseInsensitive) {
                    $word = $upperCase ? strtoupper($word) : strtolower($word);
                }
                $excludeList[] = $word;
            }
        }
        $excludeList = array_unique($excludeList);

        $wordsByScale = array();

        $index = 0;
        foreach ($words as $key => $tuple) {
            if (in_array($tuple['word'], $excludeList)) {
                continue;
            }

            $c = $tuple['count'] ? $tuple['count'] : 1;
            $s = self::$scale($c, $max);
            $size = round($minSize + abs($maxSize - $minSize) * $s, 2);
            $tuple['size'] = $size;

            $tuple['color'] = $this->$colorScale($index, $size, $maxSize);
            $tuple['href'] = $this->_getWordUrl($tuple);

            $t = array();
            foreach ($csvStruct as $k) {
                if (isset($tuple[$k]) && '' != $tuple[$k]) {
                    $t[$k] = $tuple[$k];
                }
            }
            $sKey = "$s";
            if (!is_array($wordsByScale[$sKey])) {
                $wordsByScale[$sKey] = array();
            }
            $wordsByScale[$sKey][] = $t;
            $w[] = $t;
            $index++;
        }

        $maxWords = $this->maxWords;
        if (!$maxWords) {
            return $w;
        }

        $sortedScales = array_keys($wordsByScale);
        rsort($sortedScales);

        foreach ($sortedScales as $s) {
            if (count($wordList) >= $maxWords) {
                break;
            }

            $list = $wordsByScale[$s];

            $i = 0;
            while (count($wordList) <= $maxWords && $i < count($list)) {
                $wordList[] = $list[$i];
                $i++;
            }
        }

        return $wordList;
    }

    protected static $_csvStruct = null;

    public static function getCSVStruct()
    {
        return array_keys(self::getCSVEditorStruct());
    }

    public static function getCSVEditorStruct()
    {
        // TODO: validation through editrules (required Vs not required, must support '')
        if (is_null(self::$_csvStruct)) {
            $s = array();
            $s['word']  = array('name' => t("Word"), 'editrules' => '{required: false}');
            $s['size']  = array('name' => t("Size"), 'editrules' => '{number: true}', 'width' => '50px');
            $s['href']  = array('name' => t("Link"), 'editrules' => '{/*url: true, */required: false}');
            $s['color'] = array('name' => t('Color'),'editrules' => '{required: false}', 'width' => '110px', 'resizable' => 'false',
                                'formatter' => '"color"', 'fixed' =>  'true', 'edittype' => 'custom', 'editoptions' => 'jqGridColor.editoptions()');
            self::$_csvStruct = $s;
        }

        return self::$_csvStruct;
    }

    protected function _wordsToCSV($words)
    {
        $csv = '';

        $csvStruct = self::getCSVStruct();
        foreach ($words as $tuple) {
            $l = array();
            foreach ($csvStruct as $key) {
                $l[] = isset($tuple[$key]) ? $tuple[$key] : '';
            }
            $csv .= "'".implode($l, "', '")."'\n";
        }

        return $csv;
    }

    protected function _csvToWords($csv)
    {
        $words = array();
        $csvStruct = self::getCSVStruct();
        foreach (preg_split("/\n/", $csv) as $line) {
            $csvSplit = preg_split("/,/", $line);
            $tuple = array();
            foreach ($csvSplit as $idx => $val) {
                $val = trim($val, " \t\n\r\0\x0B'\"");
                if ('' == $val) {
                    continue;
                }
                $tuple[$csvStruct[$idx]] =  stripslashes($val);
            }
            if ('' == $tuple['word']) {
                continue;
            }
            $words[] = $tuple;
        }

        return $words;
    }

    public function getWords()
    {
        if (is_null($this->_wordList)) {
            $this->_wordList = $this->_getWordList();
        }

        return $this->_wordList;
    }

    public function add()
    {
        $this->edit();
    }
    public function edit()
    {
        $this->requireAsset('core/colorpicker');
        $this->requireAsset('jqGrid');
        $this->requireAsset('jqGrid/spectrum');
        $this->requireAsset('d3.js/local');
        $this->requireAsset('gnt-wordcloudify');
    }

    public function view()
    {
        if (0 != $this->loadD3js) {
            if (0 != $this->useLocalD3Js) {
                $this->requireAsset('d3.js/local');
            } else {
                $this->requireAsset('d3.js/cdn');
            }
        }

        $this->requireAsset('gnt-wordcloudify');

        $this->set('wordList', $this->getWords());
    }

    public function getJavaScriptStrings()
    {
        return array(
            'invalid-color' => t('Please enter a valid color'),
            'invalid-float' => t('Please enter a valid decimal number'),
            'cancel'        => t('Cancel'),
            'choose'        => t('Choose'),
            'clear'            => t('Clear'),
        );
    }
}

// vim: set expandtab ts=4 :

