<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2014 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntWordCloudify\Block\GntWordCloudify;

defined('C5_EXECUTE') or die('Access Denied.');

use Core;
use Package;
use Concrete\Core\Page\Type\Type as PageType;
use View;

$form  = \Core::make('helper/form');
$ps    = \Core::make('helper/form/page_selector');
$color = \Core::make('helper/form/color');

$searchTypeOptions   = Controller::getSearchTypes();
$wordsSourceOptions  = Controller::getWordSources();
$scaleList           = Controller::getScaleList();

$underSelect = array(Controller::SEARCH_UNDER,  Controller::SEARCH_ALL_UNDER, Controller::SEARCH_SPECIFIC_PAGE);
$manualTags  = array(Controller::SOURCE_MANUAL, Controller::SOURCE_MANUAL_ADVANCED);
$showPageSel = in_array($searchType, $underSelect);
$noPageTypes = array(Controller::SEARCH_CURRENT_PAGE, Controller::SEARCH_SPECIFIC_PAGE);
$hidePageTypes = in_array($searchType, $noPageTypes);

$manual = in_array($wordsSource, $manualTags);

$pageTypeOptions = array('0' => 'All');

foreach (PageType::getList() as $pt) {
    $pageTypeOptions[$pt->getPageTypeID()] = $pt->getPageTypeName();
}

$uh = \Core::make('helper/concrete/urls');
$pkg = Package::getByHandle('gnt_word_cloudify');
$ppath = $uh->getPackageURL($pkg);

$csvStruct     = Controller::getCSVEditorStruct();
$colorTypes    = Controller::getColoringList();
$useSecondColor = array(Controller::COLORING_GRADIENT, Controller::COLORING_BICOLOR);
$useColorList = array(Controller::COLORING_MONOCHROME);
$useColorList = array_merge($useColorList, $useSecondColor);
$colorScaleList = Controller::getColoringScales();

$hideSecondColor = !in_array($coloringType, $useSecondColor);
$hideColors = !in_array($coloringType, $useColorList);

$redirectTypes = Controller::getRedirectTypes();
$targetTypes = Controller::getTargetTypes();
// advanced conf :

$advancedConfig = array(
    'loadEffect'      => array(
        'label'     => t("Load Effect"),
    ),
    'loadD3js'      => array(
        'label'     => t("Load D3.js"),
        'onchecked' => t("Unselect this only if you know what you are doing"),
        'unchecked' => t("d3js must be loaded if you want the word cloud effect!"),
        'hide'        => $loadEffect == 0,
    ),
    'useLocalD3Js' => array(
        'label'     => t("Use local d3.js"),
        'onchecked' => t("using local version defeats browser cache sharing of d3.js"),
        'hide'        => ($loadD3js == 0) || ($loadEffect == 0),
    ),
    'precalc' => array(
        'label'     => t("pre-calculate"),
        'onchecked' => t("Precalculating ensures IE8 gets the colors/sizes but may increase server load"),
        'unchecked' => t('non compatible browser (ie: IE8), will not get colors/sizes, but server load will decrease'),
    ),
    'shuffle' => array('label'     => t("shuffle words")),
    'cacheTags' => array('label'   => t('cache tags'), 'onchecked' => t("Tags will only be computed when this block is saved")),
    'enforceSize' => array('label' => t("enforce block size"), 'onchecked' => t("This only have effects for non svg compliant browsers")),

);

?>
    <style type="text/css" >
        .word-source label.control-label { text-align: right; }
        .behavior label.control-label { text-align: right; }
        .advanced label.control-label { text-align: right; }
        .display .form-group > label.control-label { text-align: right; }

        .word-source .max-words label.control-label { text-align: inherit; }

        .color-scale-box {
            display:inline-block;
            height:16px;
            margin:0px;
            padding:0px;
        }
        

    </style>

    <script type="text/javascript">

        // JS configuration :

        var jqGridOptions = {};

        jqGridOptions.colNames = [];
        jqGridOptions.colModel = [];
        jqGridOptions.caption = <?php echo json_encode(t('Tag Array'))?>;

        <?php  foreach($csvStruct as $key => $d) { ?>

            jqGridOptions.colNames.push(<?php echo json_encode($d['name'])?>);
            jqGridOptions.colModel.push({ name: "<?php echo $key?>", index: "<?php echo $key?>"<?php 

                $strings = array ('fixed', 'resizable', 'width', 'edittype');
                $jsOpts  = array ('editrules', 'editoptions', 'formatter', 'unformat');

                foreach(array_merge($strings, $jsOpts) as $cKey) {
                    if (isset($d[$cKey])) {
                        $quote = '';
                        if (in_array($cKey, $strings)) {
                            $quote = "'";
                        }

                        echo ",$cKey: $quote" . $d[$cKey] . "$quote ";
                    }
                }
            ?> });
        <?php  } ?>

        // TODO: internal formatters not appropriate... ??
        jqGridOptions.colNames.push('');
        jqGridOptions.colModel.push({name: 'actions', width: 40, fixed: true, sortable: false, resize: false, formatter: 'reactions', editable: false});

        var MANUAL_WORDSOURCES      = [ <?php echo implode($manualTags, ",")?> ];
        var SOURCE_MANUAL           = <?php echo Controller::SOURCE_MANUAL ?>;
        var SOURCE_MANUAL_ADVANCED  = <?php echo Controller::SOURCE_MANUAL_ADVANCED ?>;
        var SEARCH_UNDER_LIST       = [ <?php echo implode($underSelect, ", ")?> ];
        var NO_PAGE_TYPES           = [ <?php echo implode($noPageTypes, ", ")?> ];

        var USE_COLOR_LIST          = [ "<?php echo implode($useColorList, '", "')?>" ];
        var USE_SECOND_COLOR_LIST   = [ "<?php echo implode($useSecondColor, '", "')?>" ];
        var REDIRECT_TO_PAGE        = <?php echo Controller::REDIRECT_TO_PAGE?>;
        var TARGET_SEARCH_CUSTOM    = <?php echo Controller::TARGET_SEARCH_CUSTOM?>;
        var TARGET_SEARCH_NOSEARCH  = <?php echo Controller::TARGET_SEARCH_NOSEARCH?>;
        var COLORING_GRADIENT       = "<?php echo Controller::COLORING_GRADIENT?>";

        $(function(){gnt_cloudify_init_form(); });
    </script>

    <ul id="cloudify-tab-menu" class="nav nav-tabs"  role="tablist">
        <li class="active"><a href="#source"   role="tab" data-toggle="tab" ><?php echo t("Tags Source")     ?></a></li>
        <li               ><a href="#behavior" role="tab" data-toggle="tab" ><?php echo t("Behavior Options")?></a></li>
        <li               ><a href="#display"  role="tab" data-toggle="tab" ><?php echo t("Display Options") ?></a></li>
        <li               ><a href="#advanced" role="tab" data-toggle="tab" ><?php echo t("Advanced Options")?></a></li>
    </ul>

    <!-- Word Source -->
    <div id="cloudify-tabs" class="tab-content">

        <div class="tab-pane active word-source" id="source">

            <div class="form-group max-words" style="line-height:34px">

                <?php echo $form->label('maxWords', t("Get"), array('class' => 'col-sm-1')) ?>
                <div class="col-sm-2"><?php echo $form->number('maxWords', $maxWords) ?></div>
                <?php echo $form->label('wordsSource', t(" tags from "), array('class' => 'col-sm-2')) ?>
                <div class="input col-sm-7">
                    <?php echo $form->select('wordsSource', $wordsSourceOptions, $wordsSource) ?>
                </div>
                <div class="help-block col-sm-12"><?php echo t("Leave the number of tags empty to show all words"); ?></div>
            </div>

            <div id="cloudify-pagesearch-options" <?php echo $manual ? 'style="display:none"' : ''?>>

            <div class="form-group">
                <?php echo $form->label('searchType', t("Search"), array('class' => 'col-sm-3')) ?>
                <div class="input col-sm-9">
                    <?php echo $form->select('searchType', $searchTypeOptions, $searchType) ?>
                    <div id="cloudify-page-selector" <?php echo !$showPageSel ? 'style="display:none"' : ''?> >
                    <?php echo $ps->selectPage('baseNode', $baseNode); ?>
                    </div>
                </div>
            </div>

            <div class="form-group" id="cloudify-page-types-filter" <?php echo $hidePageTypes ? 'style="display:none"' : '' ?> >
                <?php echo $form->label('filterPageType', t('Page Types'), array('class' => 'col-sm-3')); ?>
                <div class="input col-sm-9"><?php echo $form->select('filterPageType', $pageTypeOptions, $filterPageType) ?></div>
            </div>

            </div>
            <div id="cloudify-manual-words" <?php echo !$manual ? 'style="display:none"' : ''?>>

                <div class="form-group" id="cloudify-manual-word-list" <?php echo $wordsSource != Controller::SOURCE_MANUAL ? 'style="display:none"' : '' ?>>
                    <?php echo $form->label('words', t("Words"), array('class' => 'col-sm-3')) ?>
                    <div class="input col-sm-9">
                    <?php echo $form->textarea('words', $words, array( 'style' => 'height:200px;font-size:10px' )) ?>
                    </div>
                </div>

                <div class="form-grouprtext-center" id="cloudify-advanced-tag-list" style="clear:both; margin-bottom: 10px; <?php  if($wordsSource == Controller::SOURCE_MANUAL) {?>display:none<?php  } ?>" >
                    <?php echo $form->textarea('tags', $tags, array( 'style' => 'display:none' )) ?>
                    <table id="cloudify-tags-advanced"><tr><td></td></tr></table>
                </div>
            </div>

            <div class="form-group">
                <?php echo $form->label('excludeList', t("Exclude List:"), array('class' => 'col-sm-3')) ?>
                <div class="input col-sm-9">
                    <?php echo $form->textarea('excludeList', $excludeList, array( 'style' => 'height:50px;font-size:10px' )) ?>
                    <div class="help-block"><?php echo t("Comma separated list of words to exclude"); ?></div>
                </div>
            </div>

            <script type="text/javascript">
                $(function(){
                    initTagGrid();
                });
            </script>


        </div>

        <!-- Behavior Options -->
        <div class="tab-pane behavior" id="behavior">

            <div class="form-group">
                <?php echo $form->label('redirect', t("Redirect on Click:"), array('class' => 'col-sm-3')); ?>
                <div class="input col-sm-9">
                    <?php echo $form->checkbox('redirect', 1, $redirect);?>
                </div>
            </div>

            <div id="behavior-redirect" style="clear:both;<?php echo $redirect ? '' : 'display:none;'?>">

                <div class="form-group" style="clear:both">
                    <?php echo $form->label('redirectType', t("Redirect To:"), array('class' => 'col-sm-3')) ?>
                    <div class="input col-sm-9"><?php echo $form->select('redirectType', $redirectTypes, $redirectType) ?></div>
                </div>

                <div class="form-group" id="behavior-redirect-page" style="<?php echo Controller::REDIRECT_TO_PAGE == $redirectType ? '' : 'display:none'?>" >
                    <?php echo $form->label('targetPage', t("Redirect To :"), array('class' => 'col-sm-3')) ?>
                    <div class="input col-sm-9">
                        <?php echo $ps->selectPage('targetPage', $targetPage); ?>
                    </div>
                </div>

                <div class="form-group" id="behavior-redirect-link" style="<?php echo Controller::REDIRECT_TO_URL == $redirectType ? '' : 'display:none'?>">
                    <?php echo $form->label('link', t("URL :"), array('class' => 'col-sm-3')) ?>
                    <div class="input col-sm-9"><?php echo $form->text('link', $link); ?></div>
                </div>

                <div class="form-group">
                    <?php echo $form->label('targetType', t("Send word to:"), array('class' => 'col-sm-3')) ?>
                    <div class="input col-sm-9">
                        <?php echo $form->select('targetType', $targetTypes, $targetType) ?>
                    </div>
                </div>

                <div class="form-group" id="add-custom-variable" style="clear:both;<?php echo !in_array($targetType, array(Controller::TARGET_SEARCH_CUSTOM, Controller::TARGET_SEARCH_NOSEARCH )) ? '' : 'display:none'?>">
                    <?php echo $form->label('addCustomVar', t('Add custom variable'), array('class' => 'col-sm-3'))?>
                    <div class="input col-sm-9">
                        <?php echo $form->checkbox('addCustomVar', 1, $addCustomVar); ?>
                    </div>
                </div>


                <div class="form-group" id="behavior-custom-var" style="clear:both;<?php echo Controller::TARGET_SEARCH_CUSTOM == $targetType || $addCustomVar ? '' : 'display:none'?>">
                    <?php echo $form->label('customVar', t("custom variable :"), array('class' => 'col-sm-3')) ?>
                    <div class="input col-sm-9"><?php echo $form->text('customVar', $customVar); ?></div>
                </div>

            </div>
        </div>

        <!-- Display -->
        <div class="tab-pane display" id="display">
            <div class="form-group">
                <?php echo $form->label('caseInsensitive', t("case insensitive :"), array( 'class' => 'col-sm-3')); ?>
                <div class="col-sm-1"><?php echo $form->checkbox('caseInsensitive', 1, $caseInsensitive);?></div>

                <div  class="col-sm-8" id="word-cloudify-uppercase" style="<?php echo  $caseInsensitive ? '' : 'display:none' ?>">
                    <?php echo $form->label('uppercaseWords', t("(make words upper case : ")); ?>
                    <?php echo $form->checkbox('uppercaseWords', 1, $uppercaseWords);?>)
                </div>

            </div>

            <div class="form-group">
                <?php echo $form->label('scale', t("Scale :"), array('class' => 'col-sm-3')) ?>
                <div class="col-sm-9"> <?php echo $form->select('scale', $scaleList, $scale) ?> </div>
            </div>

            <div class="form-group">
                <?php echo $form->label('width', t('Geometry'), array('class' => 'col-sm-3')); ?>
                <div class="col-sm-9 container">
                <div class="input-group col-sm-5" style="float:left">
                    <?php echo $form->number('width', $width);?>
                    <div class="input-group-addon"><?php echo t('px')?></div>
                </div>
                <div class="col-sm-2" style="text-align:center"><?php echo t('x')?></div>
                <div class="input-group col-sm-5" style="float:left">
                    <?php echo  $form->number('height', $height);?>
                    <div class="input-group-addon"><?php echo t('px')?></div>
                </div>
                </div>
            </div>

            <div class="form-group">
                <?php echo  $form->label('minSize', t('font size'), array('class' => 'col-sm-3' )); ?>
                <div class="col-sm-9 container">
                    <div class="col-sm-5 input-group" style="float:left">
                        <?php echo  $form->number('minSize', $minSize);?>
                        <div class="input-group-addon"><?php echo t('px')?></div>
                    </div>
                    <div class="col-sm-2" style="text-align: center" ><?php echo t('to')?></div>
                    <div class="col-sm-5 input-group" style="float:left">
                        <?php echo  $form->number('maxSize', $maxSize);?>
                        <div class="input-group-addon"><?php echo t('px')?></div>
                    </div>
                </div>
            </div>


            <div class="form-group">
                <?php echo  $form->label('orientationsNumbers', t('Orientations'), array('class' => 'col-sm-3')); ?>
                <div class="col-sm-9 container">
                    <div class="col-sm-3 input-group" style="float:left">
                        <?php echo  $form->number('orientationsNumbers', $orientationsNumbers);?>
                        <div class="input-group-addon"><?php echo t('steps')?></div>
                    </div>

                    <?php echo $form->label('minAngle', t('from'), array('class' => 'col-sm-2'))?>

                    <div class="col-sm-3 input-group" style="float:left">
                        <?php echo $form->number('minAngle', $minAngle)?>
                        <div class="input-group-addon"><?php echo t('°')?></div>
                    </div>

                    <?php echo $form->label('maxAngle', t('to'), array('class' => 'col-sm-1'))?>

                    <div class="col-sm-3 input-group" style="float:left">
                        <?php echo $form->number('maxAngle', $maxAngle)?>
                        <div class="input-group-addon"><?php echo t('°')?></div>
                    </div>
                </div>
            </div>

            <div class="form-group">
                <?php echo $form->label('coloringType', t("Coloring:"), array('class' => 'col-sm-3' )) ?>


                <div class="col-sm-9 container">

                    <?php  
                    foreach (array_keys($colorTypes) as $scale) {
                    $colors = Controller::getScaleColors($scale);
                    if (0 == count($colors)) {
                        continue;
                    }
                    $colorCount = count($colors);
                    $w = 488;
                    ?>
                        <div id ="word-cloudify-colors-<?php echo $scale?>" class="word-cloudify-color-preview"
                            style="width:<?php echo $w+4?>px; padding:1px;height:20px; border:1px solid; black;margin-top:1ex; margin-bottom: 1ex;<?php echo $scale != $coloringType ? 'display:none' : ''?>" >
                            <?php  
                            $idx = 0;
                            foreach ($colors as $c) { // Note: we need no space here, cause space takes ... well ... space (in term of px) and add white between the boxes
                            ?><div id="color-preview-<?php echo $scale?>-<?php echo $idx++?>" class="color-scale-box"
                                style="width:<?php echo $w/$colorCount?>px;background-color:<?php echo $c?>;">&nbsp;</div><?php 
                            } ?>
                        </div>
                    <?php  } ?>

                    <div class="col-sm-4" style="padding:0"><?php echo $form->select('coloringType', $colorTypes, $coloringType) ?></div>
                    <div class="col-sm-4" style="text-align: center"><?php echo $form->label('scaleType', t("Following :")) ?></div>
                    <div class="col-sm-4" style="padding:0"><?php echo $form->select('scaleType', $colorScaleList, $scaleType) ?></div>

                </div>
            </div>

            <div id="word-cloudify-colors" <?php echo $hideColors ? 'style="display:none"' : ''?>  class="form-group">
                <div class="col-sm-3"></div>
                <div class="col-sm-9 container">
                    <div class="col-sm-3" style="padding:0">
                        <?php echo $form->label('gradientStart', t('First Color'))?>
                    </div>
                    <div class="col-sm-2" style="padding:0">
                        <?php echo $color->output('gradientStart', $gradientStart, array('preferredFormat' => 'hex')); ?>
                    </div>
                    <div  class="col-sm-3 word-cloudify-second-color" style="padding:0; <?php echo $hideSecondColor ? 'display:none' : '' ?>" >
                        <?php echo $form->label('gradientEnd', t('Second Color'));?>
                    </div>
                    <div class="col-sm-2 word-cloudify-second-color" style="padding:0; <?php echo $hideSecondColor ? 'display:none' : '' ?>">
                        <?php echo $color->output('gradientEnd', $gradientEnd, array('preferredFormat' => 'hex')); ?>
                    </div>
                </div>
            </div>

            <!-- TODO: spiral type? !-->


        </div>

        <!-- advanced -->
        <div class="tab-pane advanced" id="advanced">

            <?php  foreach ($advancedConfig as $key => $desc) { ?>

                <div class="form-group" id="cloudify-advanced-<?php echo $key?>" style="clear:both; <?php echo $desc['hide'] ? 'display:none' : '' ?>" >
                    <?php echo $form->label($key, $desc['label'], array('class' => 'col-sm-3')); ?>
                    <div class="col-sm-9">
                        <?php echo  $form->checkbox($key, 1, ${$key}); ?>
                        <?php  if ($desc['onchecked']) { ?>
                            <div class="help-block onchecked" <?php echo  ${$key} ? '' : 'style="display:none"' ?> ><?php echo $desc['onchecked']?></div>
                        <?php  } ?>
                        <?php  if ($desc['unchecked']) { ?>
                            <div class="help-block unchecked" <?php echo  !${$key} ? '' : 'style="display:none"' ?> ><?php echo $desc['unchecked']?></div>
                        <?php  } ?>
                    </div>
                </div>

            <?php  } ?>
        </div>

    </div>
<?php 
// vim: set expandtab ts=4 spelllang=en :
