<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */

namespace Concrete\Package\GntWordCloudify;

use Concrete\Core\Localization\Localization;

defined('C5_EXECUTE') or die("Access Denied.");

use Package;
use BlockType;
use Loader;
use Events;
use Page;
use AssetList;
use Concrete\Package\GntWordCloudify\Src\AssetButler;
use Concrete\Core\Http\ResponseAssetGroup;
use Log;

class Controller extends Package
{

    protected $pkgHandle = 'gnt_word_cloudify';
    protected $appVersionRequired = "5.7.0.4";
    protected $pkgVersion = "2.0.2";

    public function getPackageDescription()
    {
        return t("Create an svg tag cloud list");
    }

    public function GetPackageName()
    {
        return t("Word Cloudify");
    }

    public function install()
    {
        $pkg = parent::install();

        BlockType::installBlockTypeFromPackage("gnt_word_cloudify", $pkg);
    }

    public function uninstall()
    {
        parent::uninstall();

        $db = Loader::db();
        $db->Execute('DROP TABLE IF EXISTS btGntWordCloudify');
    }

    public function on_start()
    {
        $ieCond = array('maxVer' => 9);
        $version = $this->getPackageVersion();

        // TODO: allow combination/minification (must test)
        $butler = new AssetButler(array('minify'=> false, 'combine' => false));

        $locale = 'en';
        $current = strtolower(Localization::getInstance()->activeLanguage());
        $jsfile = $this->getPackagePath() . "/js/3rdparty/jqgrid/js/i18n/grid.locale-${current}.js";
        if (file_exists($jsfile)) {
            $locale = $current;
        }

        $assetTree = array ( 
            'd3.js/local' => array(
                'd3.js/local' => array ('javascript', 'js/3rdparty/d3.v3.min.js', array('version' => '3.5.5'), $this, $ieCond),
            ),
            'd3.js/cdn' => array(
                // hum, should we check this version periodically?
                'd3.js/cdn' => array ('javascript', 'http://d3js.org/d3.v3.min.js', array('version' => '3.5.5', 'local' => false), $this, $ieCond),
            ),
            'gnt-wordcloudify' => array(
                'd3.layout.cloud' => array('javascript', 'js/3rdparty/d3.layout.cloud.js', array('version' => '20130412'), $this, $ieCond),
                'gnt-wordcloudify' => array('javascript', 'js/wordcloudify.js', array('version' => $version), $this),
            ),
            'jqGrid' => array(
                'jqGrid/ui.jqgrid.css' => array('css', 'js/3rdparty/jqgrid/css/ui.jqgrid.css', array('version' => "4.8.0"), $this),
                'jqGrid/concrete5.css' => array('css', 'js/jqgrid.concrete5.css', array('version' => $version), $this),
                'jqGrid/grid.locale.js' => array('javascript', "js/3rdparty/jqgrid/js/i18n/grid.locale-${locale}.js", array('version' => "4.8.0"), $this),
                'jqGrid/jquery.jGrid.min.js' => array('javascript', 'js/3rdparty/jqgrid/js/jquery.jqGrid.min.js', array('version' => "4.8.0"), $this),
            ),
            'jqGrid/spectrum' => array(
                'jqGrid/spectrum.css' => array('css', 'js/jqgrid.spectrum.css', array('version' => $version), $this),
                'jqGrid/spectrum.js' => array('javascript', 'js/jqgrid.spectrum.js', array('version' => $version), $this),
                'jqGrid/reactions.js' => array('javascript', 'js/jqgrid.reactions.js', array('version' => $version), $this),
            ),
        );

        $butler->registerAssetTree($assetTree);
    }
}

// vim: set expandtab ts=4 :

