/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2013 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */

$(function($)
{
    var coloringTypes = {
        gradient    : function(s,e) {return d3.scale.linear().range([s,e]);},
        gradientlog : function(s,e) {return d3.scale.log().range([s,e]);},
        bicolor     : function(s,e) {return d3.scale.ordinal().range([s,e]);},
        monochrome  : function(s,e) {return d3.scale.ordinal().range([s]);},
        category10  : function(s,e) {return d3.scale.category10();},
        category20  : function(s,e) {return d3.scale.category20();},
        category20b : function(s,e) {return d3.scale.category20b();},
        category20c : function(s,e) {return d3.scale.category20c();},
    };

    var methods = {

        init: function (options)
        {
            $this = $(this);

            var styles = {};
            if ($this.attr('style'))
            {
                var ss = $this.attr('style').split(';');
                $(ss).each(function(i,e) {
                    sp = this.split(':');
                    if (sp.length < 2) {
                        return;
                    }
                    styles[sp[0].trim()] = sp[1].trim();
                });
            }

            this.settings = $.extend({
                width: styles['width'] ? parseFloat( styles['width'] ) : this.width(),
                height: styles['height'] ? parseFloat( styles['height'] ) : 300,
                coloringType : 'category20',
                colorScale   : 'ordinal', // ordinal/size
                startColor: '#f00',
                endColor: '#ff0',
                angleMin: -60,
                angleMax: 60,
                steps: 5,
                targetLink: null,

            }, options);

            var words = methods.extractWords.apply(this);

            var width  = this.settings['width'];
            var height = this.settings['height'];
            var id     = this.attr('id');

            var fill = coloringTypes.category20(); // default

            var type = this.settings['coloringType'];
            if ('inherit' == type) {
                var color = this.css('color');
                fill = function (s, e) {
                    return color;
                }
            } else {
                if ('string' === typeof(type) && 'function' === typeof(coloringTypes[type]))
                {
                    fill = coloringTypes[type](this.settings['startColor'],this.settings['endColor']);
                }

                var colorScale = this.settings['colorScale'];
                if ('ordinal' == colorScale) {
                    fill.domain([0,20])
                } else if ('size' == colorScale) {
                    var min = +Infinity;
                    var max = -Infinity;

                    $.each( words, function() { 
                        if (this['size'] < min) {
                            min = this['size'];
                        }
                        if (this['size'] > max) {
                            max = this['size'];
                        }
                    });

                    if ( min != -Infinity ) {
                        fill.domain( [min,max] );
                    }
                }
            }

            var globalLink = this.settings.targetLink;


            function draw(words) 
            {

               $this = $("#"+id);
                //$this.fadeOut(10);

              // Create the svg file
               d3.select("#" + id ).append("svg").attr("width", width).attr("height", height )
              .append("g").attr("transform", "translate(" + (width/2) + "," + (height/2) + ")")
              .selectAll("text")
                .data(words)
              .enter().append("text")
                .style("font-size", function(d) {
                    return d.size + "px";
                })
                .style("font-family", "Impact")
                .style("fill", function(d, i) {
                    if ('undefined' != typeof(d.color)) {
                        return d.color;
                    }
                    if ('ordinal' == colorScale) {
                        return fill(i);
                    }
                    return fill(d.size); 
                })
                .attr("text-anchor", "middle")
                .attr("transform", function(d) {
                    return "translate(" + [d.x, d.y] + ") rotate(" + d.rotate + ")";
                })
                .text(function(d) {
                    return d.text;
                })
                .on('click', function(d) {
                    if ('function' == typeof(globalLink)) {
                        globalLink(d);
                        return;
                    }

                    var link = 'undefined' != typeof(d.href) ? d.href : globalLink;

                    if (!link) {
                        return;
                    }

                    link = link.replace(/%word%/, d.text);

                    if (link.search( /^javascript:/) == 0) {

                        var code = link.replace(/^javascript:/, '');
                        try {
                            eval(code);
                        } catch (e) {}
                        return;
                    }

                    window.location.href = link;
                })
                .style('cursor', 'pointer');
            }

            var minAngle = parseFloat(this.settings.minAngle);
            var maxAngle = parseFloat(this.settings.maxAngle);
            var steps  = parseFloat(this.settings.steps);
            if (steps == 0) {
                steps = 1.0;
            }
            var angleStep = (maxAngle - minAngle) / steps;

            d3.layout.cloud().size([width, height])
              .words(words)
              .rotate(function() {
                return ~~(Math.random() * steps) * angleStep + minAngle; 
              })
              .font("Impact")
              .fontSize(function(d) {
                return d.size;
              })
              .on("end", draw)
              .start();
        },

        extractWords: function() {
            var words = [];
            this.children('.words-data').children().each(function(idx, elem) {
                var obj = {};

                $this = $(this);

                var styles = {};
                if ($this.attr('style')) {
                    var ss = $this.attr('style').split(';');
                    $(ss).each(function(i,e){
                        sp = this.split(':');
                        if ( sp.length < 2 ) {
                            return;
                        }
                        styles[sp[0].trim()] = sp[1].trim();
                    });
                }

                obj['text'] = $this.text();

                if (styles['font-size']) {
                    obj['size'] = parseFloat( styles['font-size'] );
                }

                if (styles['color']) {
                    obj['color'] = styles['color'];
                }

                if ($this.attr('href')) {
                    obj['href'] = $this.attr('href');
                }

                words.push(obj);
            }
            );
            return words;
        }
    };

    $.fn.wordCloudify = function(method) {

        if ('undefined' == typeof(d3)) {
            if (console) {
                console.log("d3.js is not loaded, can not start");
            }
            return;
        }

        // spiral: archimedean rectangular
        // layout.font( "font" ) sets font
        // layout.spiral( 'spiral' ) sets spiral
        // 

        // Method calling logic (ala jQuery :)
        if ('string' === typeof(method) && 'function' === typeof(methods[method])) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof method === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
          $.error('Method ' +  method + ' does not exist on jQuery.wordCloudify');
        }    
    }

}(jQuery)
);

// vim: set ft=javascript sw=4 ai ci expandtab :
