<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntWordCloudify\Src;

defined('C5_EXECUTE') or die('Access Denied.');

use AssetList;
use Concrete\Core\Asset\JavascriptAsset;
use Concrete\Core\Asset\Asset;
use Concrete\Core\Foundation\Object as Object;
use Concrete\Core\Package\Package;
use Log;

class AssetButler
{
    public $defaults = array(
        'position' => false,
        'local' => true,
        'version' => false,
        'combine' => -1,
        'minify' => -1 // use the asset default
    );

    public function __construct($defaults = array())
    {
        $this->mergeDefaults($defaults);
    }

    public function mergeDefaults(array $defaults)
    {
        $this->defaults = array_merge($this->defaults, $defaults);
    }

    public function register($assetType, $assetHandle, $filename, $args = array(), $pkg = false, $cond = false)
    {
        // overload AssetList::register, with improvements:
        // - allow a Conditional decorator
		// - overwrite all the defaults (per butler)

		$args = array_merge($this->defaults, $args);

        $al = AssetList::getInstance();

        $o = $al->register($assetType, $assetHandle, $filename, $args, $pkg);

        if (!$cond) {
            return $o;
        }

        // a bit hacky... yet no really fine way to do it:
        $o = $al->getAsset($assetType, $assetHandle); // in case another version was registered
        unset($al->assets[$assetType][$assetHandle]);

        $specific = false;
        $minVer = null;
        $maxVer = null;
        extract($cond);

        $asset = new ConditionalAsset($o, $specific, $minVer, $maxVer);

        $al->registerAsset($asset);
		return $asset;
	}

    public function registerMultiple(array $assets)
    {
        // stolen (and cleaned) from the core AssetList::registerMultiple

        foreach ($assets as $asset_handle => $asset_types) {
            foreach ($asset_types as $asset_settings) {
                array_splice($asset_settings, 1, 0, $asset_handle);
                call_user_func_array(array($this, 'register'), $asset_settings);
            }
        }
    }

    public function registerGroup()
    {
        $al = AssetList::getInstance();
        return call_user_func_array(array($al, 'registerGroup'), func_get_args());
    }

    public function registerGroups(array $groups)
    {
        foreach ($groups as $group) {
            $this->registerGroup($group);
        }
    }

    public function registerAssetTree(array $tree)
    {
        foreach ($tree as $groupHandle => $content) {
            $groupList = array();
            foreach ($content as $assetHandle => $settings) {
                $groupList[] = array($settings[0], $assetHandle);
                array_splice($settings, 1, 0, $assetHandle);
                call_user_func_array(array($this, 'register'), $settings);
            }
            $this->registerGroup($groupHandle, $groupList);
        }
    }

}
