<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntWordCloudify\Src;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Asset\Asset;

class ConditionalAsset extends Asset
{
    protected $_maxVer          = null;
    protected $_minVer          = null;
    protected $_specificVersion = null;
    protected $_asset           = null;

    public function __construct($asset, $specific = false, $minVer = null, $maxVer = null)
    {
        $this->_asset = $asset;
        parent::__construct($asset->getAssetHandle());

        $this->_minVer = $minVer;
        $this->_specificVersion = $specific;
        $this->_maxVer = $maxVer;
    }

    public function setMinimumVersion($minVer = null)
    {
        $this->_minVer = $minVer;
    }
    public function setMaximumVersion($maxVer = null)
    {
        $this->_maxVer = $maxVer;
    }

    public function __toString()
    {
        $cond = '';
        if ($this->_specificVersion) {
            $cond = "IE $this->_specificVersion";
        } else {
            $ieMinCond = $this->_minVer ? "lte IE $this->_minVer " : '';
            $ieMaxCond = $this->_maxVer ? "gte IE $this->_maxVer " : '';
            $and = ('' != $ieMinCond && '' != $ieMaxCond) ? '&& ' : '';
            $cond = "$ieMinCond$and$ieMaxCond";
        }

        $str = "<!--[ if $cond]><!-->\n";
        $str .= $this->_asset;
        $str .= "\n<!-- <![endif]-->\n";

        return $str;
    }

    // magic decorator:
    public function __call($fn, $args)
    {
        return call_user_func_array(array( $this->_asset,  $fn), $args);
    }
	// implementation of abstract methods
    public function getAssetDefaultPosition()
    {
        return $this->_asset->getAssetDefaultPosition();
    }
    public function getAssetType()
    {
        return $this->_asset->getAssetType();
    }
    public function minify($assets)
    {
        return $this->_asset->minify($assets);
    }
    public function combine($assets)
    {
        return $this->_asset->combine($assets);
    }
	public static function process($asset)
	{
		return static::process($asset);
	}
}

// vim: set noexpandtab ts=4 :

